import asyncio
import sys

from updatable import get_package_update_list, get_parsed_environment_package_list


async def get_python_env_info() -> dict:
    runtime = {
        "platform": "python",
        "platform_version": sys.version,
        "framework": "django",
        "framework_installed_version": None,
        "framework_newest_version": None,
    }
    modules = []
    packages = get_parsed_environment_package_list()

    for package in packages:
        package["_data"] = asyncio.create_task(
            get_package_update_list(package["package"], package["version"]),
        )

    for package in packages:
        package_data = await package["_data"]

        modules.append(
            {
                "name": package["package"],
                "installed_version": package["version"],
                "installed_version_licences": [
                    package_data["current_release_license"],
                ],
                "newest_version": package_data["latest_release"],
                "newest_version_licences": [
                    package_data["latest_release_license"],
                ],
            },
        )

        if package["package"] == "Django":
            runtime["framework_installed_version"] = package["version"]
            runtime["framework_newest_version"] = package_data["latest_release"]

    return {
        "runtime": runtime,
        "modules": modules,
    }



from django.urls import include, re_path

from .views import MonitorModulesView, MonitorUpView

urlpatterns = [
    # API v1
    re_path(
        r"^anxapi/v1/",
        include(
            [
                re_path(
                    r"^modules/$",
                    MonitorModulesView.as_view(),
                    name="anexia_monitor_modules",
                ),
                re_path(r"^up/$", MonitorUpView.as_view(), name="anexia_monitor_up"),
            ],
        ),
    ),
]



from io import StringIO
import json

from django.test import TestCase, Client
from django.urls import reverse
from django.conf import settings
from django.core.management import call_command

client = Client()


class MonitoringAPITests(TestCase):
    """
    Test for monitoring endpoints. The endpoints are configured in the main urls.py and come
    from the anexia_monitoring package. The access token must be defined in the django settings.
    """

    def test_monitoring_endpoint(self):
        """
        Ensure the monitoring endpoint can be called
        """
        url = reverse("anexia_monitor_modules")
        response = client.get(url, format="json")
        self.assertEqual(response.status_code, 401)

        response = client.get(
            "%s?access_token=invalid_access_token" % url,
            format="json",
        )
        self.assertEqual(response.status_code, 401)

        response = client.get(
            f"{url}?access_token={settings.ANX_MONITORING_ACCESS_TOKEN}",
            format="json",
        )
        self.assertEqual(response.status_code, 200)

    def test_up_endpoint(self):
        """
        Ensure the up endpoint can be called
        """
        url = reverse("anexia_monitor_up")
        response = client.get(url, format="json")
        self.assertEqual(response.status_code, 401)

        response = client.get(
            "%s?access_token=invalid_access_token" % url,
            format="json",
        )
        self.assertEqual(response.status_code, 401)

        response = client.get(
            f"{url}?access_token={settings.ANX_MONITORING_ACCESS_TOKEN}",
            format="json",
        )
        self.assertEqual(response.status_code, 200)


class MngmtCommandModulesTests(TestCase):
    """
    Tests django management command `anxmonitormodules`.
    """

    def is_json(self, json_str):
        try:
            json.loads(json_str)
        except ValueError:
            return False
        return True

    def test_command_success(self):
        out = StringIO()
        call_command("anxmonitormodules", stdout=out)
        output = out.getvalue().strip()

        self.assertTrue(self.is_json(output))



import asyncio

from django.http import JsonResponse, HttpResponse
from django.contrib.auth import get_user_model
from django.db import connections
from django.dispatch import receiver
from django.views.generic import View
from django.conf import settings

from .core import get_python_env_info
from .decorators import access_token_check
from .events import monitor_up_check


class BaseView(View):
    """
    Base view
    """

    @staticmethod
    def add_access_control_headers(response):
        """
        Adds CORS headers to http response

        :param response: HttpResponse
        :return: HttpResponse
        """
        response["Access-Control-Allow-Origin"] = "*"
        response["Access-Control-Allow-Methods"] = "GET, OPTIONS"
        return response

    def options(self, request, *args, **kwargs):
        response = HttpResponse()
        self.add_access_control_headers(response)
        return response


class MonitorModulesView(BaseView):
    """
    A view that returns a list of all modules installed on the current environment, having current and latest version
    of the module. It also contains information about the runtime (python and django version).
    """

    @access_token_check
    def get(self, request, *args, **kwargs):
        response = JsonResponse(asyncio.run(get_python_env_info()))
        self.add_access_control_headers(response)
        return response


class MonitorUpView(BaseView):
    """
    Simple view that checks if the Django is alive and working correctly

    Checks for:
      - Database connections
      - Query for users
      - Registers signal for custom check
    """

    @access_token_check
    def get(self, request, *args, **kwargs):
        response_contents = []
        response_status = 200

        # Sends signal for health check implementations
        up_check_results = monitor_up_check.send_robust(sender=None)
        up_check_result_warnings = [
            r[1]
            for r in up_check_results
            if isinstance(r[1], Exception)
            and getattr(r[1], "is_anx_monitoring_up_warning", False)
        ]
        up_check_result_errors = [
            r[1]
            for r in up_check_results
            if isinstance(r[1], Exception)
            and not getattr(r[1], "is_anx_monitoring_up_warning", False)
        ]

        if not up_check_result_warnings and not up_check_result_errors:
            response_contents.append("OK")
            response_status = 200

        for up_check_result_warning in up_check_result_warnings:
            response_contents.append(
                "WARNING: {}".format(str(up_check_result_warning) or "-"),
            )
            response_status = 200

        for up_check_result_error in up_check_result_errors:
            response_contents.append(
                "ERROR: {}".format(str(up_check_result_error) or "-"),
            )
            response_status = 500

        response = HttpResponse(
            "\n".join(response_contents),
            content_type="text/plain",
            status=response_status,
        )
        self.add_access_control_headers(response)
        return response


if getattr(settings, "ANX_MONITORING_TEST_DB_CONNECTIONS", True):

    @receiver(monitor_up_check)
    def anx_monitoring_test_db_connections(sender, **kwargs):
        for connection_key in connections:
            connections[connection_key].cursor()


if getattr(settings, "ANX_MONITORING_TEST_QUERY_USERS", True):

    @receiver(monitor_up_check)
    def anx_monitoring_test_query_users(sender, **kwargs):
        user_model = get_user_model()
        user_model.objects.all().count()



from django.conf import settings
from django.http import HttpResponse


def access_token_check(func):
    """
    Decorator that checks if access token is set and correct before calling the method or function.

    :return: Decorator
    """

    def _is_authorized(request):
        # Access token GET param must be defined
        if "access_token" not in request.request.GET:
            return False

        # Access token setting must be defined
        if not hasattr(settings, "ANX_MONITORING_ACCESS_TOKEN"):
            return False

        # Validate access token
        if settings.ANX_MONITORING_ACCESS_TOKEN != request.request.GET["access_token"]:
            return False

        return True

    def access_token_check_wrapper(request, *args, **kwargs):
        if not _is_authorized(request):
            return HttpResponse("401 Unauthorized", status=401)
        return func(request, *args, **kwargs)

    return access_token_check_wrapper



import asyncio
import json

from django.core.management.base import BaseCommand
from anexia_monitoring.core import get_python_env_info


class Command(BaseCommand):
    help = "Outputs JSON object describing python runtime and modules. Same as /anxapi/vi/modules"

    def handle(self, *args, **kwargs):
        modules_dict = asyncio.run(get_python_env_info())
        modules_json = json.dumps(modules_dict, indent=4)
        self.stdout.write(modules_json)
        self.stdout.flush()
